#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("Enter the description of your new custom indicator here")]
    public class genericoverlay : Indicator
    {
        #region Variables
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(Color.Blue, PlotStyle.Line, "Plot0"));
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			AutoScale = false;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
            Plot0.Set(150);
        }

		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			bool drawscale = true; // Limited scale marks
			double scaleHigh = 200; // Max value of indicator (100 for Stochastic, +400 for CCI, etc.)
			double scaleLow = -200; // Min value of indicator (0 for Stochastic, -400 for CCI, etc.)
			double upperline = 175; // Optional horizontal line 1 (80 for Stochastic)
			double lowerline = -175; // Optional horizontal line 2 (20 for Stochastic)
			Color uppercolor = Color.Green; // Color for line 1
			Color lowercolor = Color.Green; // Color for line 2
			
			//Draw Scale
			StringFormat stringFormat = new StringFormat();
			SolidBrush textBrush = new SolidBrush(Color.Black);
			System.Drawing.Font textFont = new Font("Arial", 8);
			double scaleRange = (scaleHigh - scaleLow);
			double scaleShift = ( scaleLow < 0 ) ? -scaleLow : 0;
			if(drawscale)
			{
				//Upper line
				double line = ((upperline+scaleShift)/scaleRange)*100;
				graphics.DrawLine(new Pen(uppercolor,2),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((double)line/100))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((double)line/100))));
				graphics.DrawString(upperline.ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((double)line/100))),stringFormat);
				
				//1/4 up the scale
				line = (scaleRange*.75)+scaleLow;
				graphics.DrawString(line.ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-0.75)),stringFormat);
				
				//Center Line
				line = ((scaleRange/2)+scaleLow)*100;
				graphics.DrawLine(new Pen(Color.DarkGray,2),bounds.X,(float)((bounds.Bottom-bounds.Y)*0.50),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*0.50));
				graphics.DrawString(((scaleRange/2)+scaleLow).ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-0.50)),stringFormat);
				
				//3/4 up the scale
				line = (scaleRange*.25)+scaleLow;
				graphics.DrawString(line.ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-0.25)),stringFormat);
				
				//Lower line
				line = ((lowerline+scaleShift)/scaleRange)*100;
				graphics.DrawLine(new Pen(lowercolor,2),bounds.X,(float)((bounds.Bottom-bounds.Y)*(1-((double)line/100))),bounds.X+bounds.Width,(float)((bounds.Bottom-bounds.Y)*(1-((double)line/100))));
				graphics.DrawString(lowerline.ToString(), textFont, textBrush, bounds.X, (float)((bounds.Bottom-bounds.Y)*(1-((double)line/100))),stringFormat);
			}
			
			//Plot Indicator
			int barWidth = ChartControl.ChartStyle.GetBarPaintWidth(ChartControl.BarWidth);
			SmoothingMode oldSmoothingMode = graphics.SmoothingMode;
			GraphicsPath dpath = new GraphicsPath();
			int lastX = -1;
			int lastY = -1;
			DataSeries series = (DataSeries) Values[0];
			double seriesvalue = 0;
			Gui.Chart.Plot plot = Plots[0];
			for(int count = 0; count < ChartControl.BarsPainted; count++)
			{
				int idx = ChartControl.LastBarPainted - ChartControl.BarsPainted + 1 + count;
				if (idx < 0 || idx >= Input.Count || (!ChartControl.ShowBarsRequired && idx < BarsRequired))
				continue;
				
				seriesvalue = ((series.Get(idx))+scaleShift)/scaleRange;
				if (seriesvalue == 0)
				continue;
				
				int x = (int) (ChartControl.CanvasRight - ChartControl.BarMarginRight - barWidth / 2 - (ChartControl.BarsPainted - 1) * ChartControl.BarSpace + count * ChartControl.BarSpace) + 1;
				int y = (int) ((bounds.Bottom-bounds.Y)*(1-seriesvalue));
				
				if (lastX >= 0)
				{
				dpath.AddLine(lastX - plot.Pen.Width / 2, lastY, x - plot.Pen.Width / 2, y);
				}
			
				lastX = x;
				lastY = y;
			}
			dpath.Reverse();
			
			graphics.SmoothingMode = SmoothingMode.AntiAlias;
			graphics.DrawPath(Plots[0].Pen, dpath);
			graphics.SmoothingMode = oldSmoothingMode;
		}
		
        #region Properties
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private genericoverlay[] cachegenericoverlay = null;

        private static genericoverlay checkgenericoverlay = new genericoverlay();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public genericoverlay genericoverlay()
        {
            return genericoverlay(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public genericoverlay genericoverlay(Data.IDataSeries input)
        {

            if (cachegenericoverlay != null)
                for (int idx = 0; idx < cachegenericoverlay.Length; idx++)
                    if (cachegenericoverlay[idx].EqualsInput(input))
                        return cachegenericoverlay[idx];

            genericoverlay indicator = new genericoverlay();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;

            genericoverlay[] tmp = new genericoverlay[cachegenericoverlay == null ? 1 : cachegenericoverlay.Length + 1];
            if (cachegenericoverlay != null)
                cachegenericoverlay.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cachegenericoverlay = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.genericoverlay genericoverlay()
        {
            return _indicator.genericoverlay(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.genericoverlay genericoverlay(Data.IDataSeries input)
        {
            return _indicator.genericoverlay(input);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.genericoverlay genericoverlay()
        {
            return _indicator.genericoverlay(Input);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.genericoverlay genericoverlay(Data.IDataSeries input)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.genericoverlay(input);
        }

    }
}
#endregion
